/**************************************************************
 *
 *  Program Title:  WaspARM monitor
 *
 *  Program Name:  ARM_Serial.c	  Version: 0.80
 *
 *  Author:  BR	  				   Date:  March 26, 2008
 *
 *
 *  This module handles serial IO
 *
 * Copyright 2009, SOC ROBOTICS, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC ROBOTICS,
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS,
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */
#include "board.h"
#include "ARM_Serial.h"

#define BRD  (MCK/(16*BAUD))                    /* Baud Rate Divisor */

static volatile AT91PS_PMC pPMC = AT91C_BASE_PMC;// pointer to PMC data structure
static volatile AT91PS_PIO pPIO = AT91C_BASE_PIOA;// pointer to PIO data structure
static volatile AT91PS_USART pUSART = AT91C_BASE_US0;

void uart0_init (void) {                   /* Initialize Serial Interface */


  pPMC->PMC_PCER = (1<<AT91C_ID_PIOA) | // enable the Parallel IO Controller clock
                   (1<<AT91C_ID_US0);   //enable the UART

  pPIO->PIO_ASR = AT91C_PA5_RXD0 |        /* PIO Peripheral A Select for RxD0 Pin */
  				  AT91C_PA6_TXD0;         /* PIO Peripheral A Select for TxD0 Pin */
  pPIO->PIO_PDR = AT91C_PA5_RXD0 |        /* Enable RxD0 Pin */
                  AT91C_PA6_TXD0;         /* Enable TxD0 Pin */

  pUSART->US_CR = AT91C_US_RSTRX |          /* Reset Receiver      */
                  AT91C_US_RSTTX |          /* Reset Transmitter   */
                  AT91C_US_RXDIS |          /* Receiver Disable    */
                  AT91C_US_TXDIS;           /* Transmitter Disable */

  pUSART->US_MR = AT91C_US_USMODE_NORMAL |  /* Normal Mode */
                  AT91C_US_CLKS_CLOCK    |  /* Clock = MCK */
                  AT91C_US_CHRL_8_BITS   |  /* 8-bit Data  */
                  AT91C_US_PAR_NONE      |  /* No Parity   */
                  AT91C_US_NBSTOP_1_BIT;    /* 1 Stop Bit  */

  pUSART->US_BRGR = BRD;                    /* Baud Rate Divisor */

  pUSART->US_CR = AT91C_US_RXEN  |          /* Receiver Enable     */
                  AT91C_US_TXEN;            /* Transmitter Enable  */
}

int uart0_putc(int ch)
{
	while (!(pUSART->US_CSR & AT91C_US_TXRDY));   /* Wait for Empty Tx Buffer */
	return (pUSART->US_THR = ch);                 /* Transmit Character */
}

/* Write Character to Serial Port */
int uart0_putchar(int ch)
{
  if(ch == '\n')  {                            /* Check for LF */
      //uart0_putc( '\r' );                         /* Output CR */
      }
  return uart0_putc( ch );                     /* Transmit Character */
}

void uart0_puts(char* s)
{
	while ( *s ) uart0_putchar( *s++ );
}

/* returns true if character in receive buffer */
int uart0_kbhit(void)
{
	if ( pUSART->US_CSR & AT91C_US_RXRDY) {
		return 1;
	}
	else {
		return 0;
	}
}

/* Read Character from Serial Port */
int uart0_getc(void)
{

  while (!(pUSART->US_CSR & AT91C_US_RXRDY));   /* Wait for Full Rx Buffer */
  return (pUSART->US_RHR);                      /* Read Character */
}
